////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Qualcomm Technologies, Inc. and/or its subsidiaries.
// All Rights Reserved.
// Confidential and Proprietary - Qualcomm Technologies, Inc.
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// THIS IS AUTO-GENERATED CODE.  PLEASE DO NOT EDIT (File bug reports against tools).
///
/// Auto-generated by: Parameter Parser V5.5.1 (2411131018)
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @file  tmc_1_4_0.cpp
/// @brief Auto-generated Chromatix parameter file
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "tmc_1_4_0.h"
#include "parametersetmanager.h"

TMC_1_4_0_NAMESPACE_BEGIN

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// SetValue/GetValue Functions
/// Gets or sets the requested value
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool tmc14_rgn_dataType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "gtm_percentage_tab") == 0)
    {
        __result = gtm_percentage_tab.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "ltm_percentage_tab") == 0)
    {
        __result = ltm_percentage_tab.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "dark_boost_ratio") == 0)
    {
        ParameterUtils::DoubleToString((double)dark_boost_ratio, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "dark_boost_offset_tab") == 0)
    {
        __result = dark_boost_offset_tab.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "tone_target_tab") == 0)
    {
        __result = tone_target_tab.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "tone_anchors_tab") == 0)
    {
        __result = tone_anchors_tab.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "scale_ratio_smooth") == 0)
    {
        ParameterUtils::DoubleToString((double)scale_ratio_smooth, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "face_sa_target") == 0)
    {
        ParameterUtils::DoubleToString((double)face_sa_target, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "face_sa_adj_max") == 0)
    {
        ParameterUtils::DoubleToString((double)face_sa_adj_max, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "face_sa_adj_smooth") == 0)
    {
        ParameterUtils::DoubleToString((double)face_sa_adj_smooth, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "face_sa_score_gain") == 0)
    {
        ParameterUtils::DoubleToString((double)face_sa_score_gain, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "face_sa_score_offset") == 0)
    {
        ParameterUtils::DoubleToString((double)face_sa_score_offset, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "tone_bright_adj") == 0)
    {
        ParameterUtils::DoubleToString((double)tone_bright_adj, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "tone_dark_adj") == 0)
    {
        ParameterUtils::DoubleToString((double)tone_dark_adj, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "stretch_bright_str") == 0)
    {
        ParameterUtils::DoubleToString((double)stretch_bright_str, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "stretch_dark_str") == 0)
    {
        ParameterUtils::DoubleToString((double)stretch_dark_str, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "hist_supr_range_start") == 0)
    {
        ParameterUtils::DoubleToString((double)hist_supr_range_start, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "hist_supr_range_end") == 0)
    {
        ParameterUtils::DoubleToString((double)hist_supr_range_end, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "hist_boost_range_start") == 0)
    {
        ParameterUtils::DoubleToString((double)hist_boost_range_start, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "hist_boost_range_end") == 0)
    {
        ParameterUtils::DoubleToString((double)hist_boost_range_end, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "hist_avg_range_start") == 0)
    {
        ParameterUtils::DoubleToString((double)hist_avg_range_start, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "hist_avg_range_end") == 0)
    {
        ParameterUtils::DoubleToString((double)hist_avg_range_end, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "hist_conv_kernel_tab") == 0)
    {
        __result = hist_conv_kernel_tab.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "hist_clip_slope") == 0)
    {
        ParameterUtils::DoubleToString((double)hist_clip_slope, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "hist_enhance_clamp") == 0)
    {
        ParameterUtils::DoubleToString((double)hist_enhance_clamp, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "hist_enhance_ratio_tab") == 0)
    {
        __result = hist_enhance_ratio_tab.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "contrast_bright_clip") == 0)
    {
        ParameterUtils::DoubleToString((double)contrast_bright_clip, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "contrast_dark_adj") == 0)
    {
        ParameterUtils::DoubleToString((double)contrast_dark_adj, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "contrast_he_bright_tab") == 0)
    {
        __result = contrast_he_bright_tab.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "contrast_he_dark_tab") == 0)
    {
        __result = contrast_he_dark_tab.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "hist_smoothing_str") == 0)
    {
        ParameterUtils::DoubleToString((double)hist_smoothing_str, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "hist_curve_smoothing_str") == 0)
    {
        ParameterUtils::DoubleToString((double)hist_curve_smoothing_str, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "scene_change_smoothing_str") == 0)
    {
        ParameterUtils::DoubleToString((double)scene_change_smoothing_str, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "scene_change_curve_smoothing_str") == 0)
    {
        ParameterUtils::DoubleToString((double)scene_change_curve_smoothing_str, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "scene_change_hist_delta_th1") == 0)
    {
        ParameterUtils::DoubleToString((double)scene_change_hist_delta_th1, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "scene_change_hist_delta_th2") == 0)
    {
        ParameterUtils::DoubleToString((double)scene_change_hist_delta_th2, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "scene_change_lux_idx_delta_th1") == 0)
    {
        ParameterUtils::DoubleToString((double)scene_change_lux_idx_delta_th1, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "scene_change_lux_idx_delta_th2") == 0)
    {
        ParameterUtils::DoubleToString((double)scene_change_lux_idx_delta_th2, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "stats_fusion_long_str") == 0)
    {
        ParameterUtils::DoubleToString((double)stats_fusion_long_str, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "stats_fusion_medium_str") == 0)
    {
        ParameterUtils::DoubleToString((double)stats_fusion_medium_str, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "stats_fusion_long_range") == 0)
    {
        ParameterUtils::DoubleToString((double)stats_fusion_long_range, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "stats_fusion_medium_range") == 0)
    {
        ParameterUtils::DoubleToString((double)stats_fusion_medium_range, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "stats_fusion_short_range") == 0)
    {
        ParameterUtils::DoubleToString((double)stats_fusion_short_range, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "adjustment_face_th1") == 0)
    {
        ParameterUtils::DoubleToString((double)adjustment_face_th1, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "adjustment_face_th2") == 0)
    {
        ParameterUtils::DoubleToString((double)adjustment_face_th2, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "adjustment_face_smoothing_str") == 0)
    {
        ParameterUtils::DoubleToString((double)adjustment_face_smoothing_str, __value, __size);
        __result = true;
    }

    return __result;
}

bool tmc14_rgn_dataType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "gtm_percentage_tab") == 0)
    {
        __result = gtm_percentage_tab.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "ltm_percentage_tab") == 0)
    {
        __result = ltm_percentage_tab.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "dark_boost_ratio") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            dark_boost_ratio = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "dark_boost_offset_tab") == 0)
    {
        __result = dark_boost_offset_tab.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "tone_target_tab") == 0)
    {
        __result = tone_target_tab.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "tone_anchors_tab") == 0)
    {
        __result = tone_anchors_tab.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "scale_ratio_smooth") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            scale_ratio_smooth = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "face_sa_target") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            face_sa_target = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "face_sa_adj_max") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            face_sa_adj_max = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "face_sa_adj_smooth") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            face_sa_adj_smooth = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "face_sa_score_gain") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            face_sa_score_gain = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "face_sa_score_offset") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            face_sa_score_offset = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "tone_bright_adj") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            tone_bright_adj = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "tone_dark_adj") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            tone_dark_adj = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "stretch_bright_str") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            stretch_bright_str = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "stretch_dark_str") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            stretch_dark_str = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "hist_supr_range_start") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            hist_supr_range_start = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "hist_supr_range_end") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            hist_supr_range_end = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "hist_boost_range_start") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            hist_boost_range_start = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "hist_boost_range_end") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            hist_boost_range_end = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "hist_avg_range_start") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            hist_avg_range_start = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "hist_avg_range_end") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            hist_avg_range_end = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "hist_conv_kernel_tab") == 0)
    {
        __result = hist_conv_kernel_tab.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "hist_clip_slope") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            hist_clip_slope = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "hist_enhance_clamp") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            hist_enhance_clamp = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "hist_enhance_ratio_tab") == 0)
    {
        __result = hist_enhance_ratio_tab.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "contrast_bright_clip") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            contrast_bright_clip = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "contrast_dark_adj") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            contrast_dark_adj = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "contrast_he_bright_tab") == 0)
    {
        __result = contrast_he_bright_tab.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "contrast_he_dark_tab") == 0)
    {
        __result = contrast_he_dark_tab.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "hist_smoothing_str") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            hist_smoothing_str = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "hist_curve_smoothing_str") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            hist_curve_smoothing_str = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "scene_change_smoothing_str") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            scene_change_smoothing_str = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "scene_change_curve_smoothing_str") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            scene_change_curve_smoothing_str = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "scene_change_hist_delta_th1") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            scene_change_hist_delta_th1 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "scene_change_hist_delta_th2") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            scene_change_hist_delta_th2 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "scene_change_lux_idx_delta_th1") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            scene_change_lux_idx_delta_th1 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "scene_change_lux_idx_delta_th2") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            scene_change_lux_idx_delta_th2 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "stats_fusion_long_str") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            stats_fusion_long_str = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "stats_fusion_medium_str") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            stats_fusion_medium_str = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "stats_fusion_long_range") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            stats_fusion_long_range = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "stats_fusion_medium_range") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            stats_fusion_medium_range = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "stats_fusion_short_range") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            stats_fusion_short_range = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "adjustment_face_th1") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            adjustment_face_th1 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "adjustment_face_th2") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            adjustment_face_th2 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "adjustment_face_smoothing_str") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            adjustment_face_smoothing_str = (float)dblValue; 
        }
    }

    return __result;
}

bool tmc14_rgn_dataType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // gtm_percentage_tab
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("gtm_percentage_tab", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "gtm_percentage_tab") == 0)
    {
        __result = gtm_percentage_tab.GetInfo(&__path[__pos], __info);
    }

    // ltm_percentage_tab
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("ltm_percentage_tab", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "ltm_percentage_tab") == 0)
    {
        __result = ltm_percentage_tab.GetInfo(&__path[__pos], __info);
    }

    // dark_boost_ratio
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("dark_boost_ratio", ParameterParserDataType::Float, (double)dark_boost_ratio));
    }
    else if (PARAMETER_STRCMP(__name, "dark_boost_ratio") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)dark_boost_ratio));
    }

    // dark_boost_offset_tab
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("dark_boost_offset_tab", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "dark_boost_offset_tab") == 0)
    {
        __result = dark_boost_offset_tab.GetInfo(&__path[__pos], __info);
    }

    // tone_target_tab
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("tone_target_tab", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "tone_target_tab") == 0)
    {
        __result = tone_target_tab.GetInfo(&__path[__pos], __info);
    }

    // tone_anchors_tab
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("tone_anchors_tab", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "tone_anchors_tab") == 0)
    {
        __result = tone_anchors_tab.GetInfo(&__path[__pos], __info);
    }

    // scale_ratio_smooth
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("scale_ratio_smooth", ParameterParserDataType::Float, (double)scale_ratio_smooth));
    }
    else if (PARAMETER_STRCMP(__name, "scale_ratio_smooth") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)scale_ratio_smooth));
    }

    // face_sa_target
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("face_sa_target", ParameterParserDataType::Float, (double)face_sa_target));
    }
    else if (PARAMETER_STRCMP(__name, "face_sa_target") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)face_sa_target));
    }

    // face_sa_adj_max
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("face_sa_adj_max", ParameterParserDataType::Float, (double)face_sa_adj_max));
    }
    else if (PARAMETER_STRCMP(__name, "face_sa_adj_max") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)face_sa_adj_max));
    }

    // face_sa_adj_smooth
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("face_sa_adj_smooth", ParameterParserDataType::Float, (double)face_sa_adj_smooth));
    }
    else if (PARAMETER_STRCMP(__name, "face_sa_adj_smooth") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)face_sa_adj_smooth));
    }

    // face_sa_score_gain
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("face_sa_score_gain", ParameterParserDataType::Float, (double)face_sa_score_gain));
    }
    else if (PARAMETER_STRCMP(__name, "face_sa_score_gain") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)face_sa_score_gain));
    }

    // face_sa_score_offset
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("face_sa_score_offset", ParameterParserDataType::Float, (double)face_sa_score_offset));
    }
    else if (PARAMETER_STRCMP(__name, "face_sa_score_offset") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)face_sa_score_offset));
    }

    // tone_bright_adj
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("tone_bright_adj", ParameterParserDataType::Float, (double)tone_bright_adj));
    }
    else if (PARAMETER_STRCMP(__name, "tone_bright_adj") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)tone_bright_adj));
    }

    // tone_dark_adj
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("tone_dark_adj", ParameterParserDataType::Float, (double)tone_dark_adj));
    }
    else if (PARAMETER_STRCMP(__name, "tone_dark_adj") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)tone_dark_adj));
    }

    // stretch_bright_str
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("stretch_bright_str", ParameterParserDataType::Float, (double)stretch_bright_str));
    }
    else if (PARAMETER_STRCMP(__name, "stretch_bright_str") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)stretch_bright_str));
    }

    // stretch_dark_str
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("stretch_dark_str", ParameterParserDataType::Float, (double)stretch_dark_str));
    }
    else if (PARAMETER_STRCMP(__name, "stretch_dark_str") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)stretch_dark_str));
    }

    // hist_supr_range_start
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("hist_supr_range_start", ParameterParserDataType::Float, (double)hist_supr_range_start));
    }
    else if (PARAMETER_STRCMP(__name, "hist_supr_range_start") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)hist_supr_range_start));
    }

    // hist_supr_range_end
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("hist_supr_range_end", ParameterParserDataType::Float, (double)hist_supr_range_end));
    }
    else if (PARAMETER_STRCMP(__name, "hist_supr_range_end") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)hist_supr_range_end));
    }

    // hist_boost_range_start
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("hist_boost_range_start", ParameterParserDataType::Float, (double)hist_boost_range_start));
    }
    else if (PARAMETER_STRCMP(__name, "hist_boost_range_start") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)hist_boost_range_start));
    }

    // hist_boost_range_end
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("hist_boost_range_end", ParameterParserDataType::Float, (double)hist_boost_range_end));
    }
    else if (PARAMETER_STRCMP(__name, "hist_boost_range_end") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)hist_boost_range_end));
    }

    // hist_avg_range_start
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("hist_avg_range_start", ParameterParserDataType::Float, (double)hist_avg_range_start));
    }
    else if (PARAMETER_STRCMP(__name, "hist_avg_range_start") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)hist_avg_range_start));
    }

    // hist_avg_range_end
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("hist_avg_range_end", ParameterParserDataType::Float, (double)hist_avg_range_end));
    }
    else if (PARAMETER_STRCMP(__name, "hist_avg_range_end") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)hist_avg_range_end));
    }

    // hist_conv_kernel_tab
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("hist_conv_kernel_tab", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "hist_conv_kernel_tab") == 0)
    {
        __result = hist_conv_kernel_tab.GetInfo(&__path[__pos], __info);
    }

    // hist_clip_slope
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("hist_clip_slope", ParameterParserDataType::Float, (double)hist_clip_slope));
    }
    else if (PARAMETER_STRCMP(__name, "hist_clip_slope") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)hist_clip_slope));
    }

    // hist_enhance_clamp
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("hist_enhance_clamp", ParameterParserDataType::Float, (double)hist_enhance_clamp));
    }
    else if (PARAMETER_STRCMP(__name, "hist_enhance_clamp") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)hist_enhance_clamp));
    }

    // hist_enhance_ratio_tab
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("hist_enhance_ratio_tab", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "hist_enhance_ratio_tab") == 0)
    {
        __result = hist_enhance_ratio_tab.GetInfo(&__path[__pos], __info);
    }

    // contrast_bright_clip
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("contrast_bright_clip", ParameterParserDataType::Float, (double)contrast_bright_clip));
    }
    else if (PARAMETER_STRCMP(__name, "contrast_bright_clip") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)contrast_bright_clip));
    }

    // contrast_dark_adj
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("contrast_dark_adj", ParameterParserDataType::Float, (double)contrast_dark_adj));
    }
    else if (PARAMETER_STRCMP(__name, "contrast_dark_adj") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)contrast_dark_adj));
    }

    // contrast_he_bright_tab
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("contrast_he_bright_tab", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "contrast_he_bright_tab") == 0)
    {
        __result = contrast_he_bright_tab.GetInfo(&__path[__pos], __info);
    }

    // contrast_he_dark_tab
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("contrast_he_dark_tab", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "contrast_he_dark_tab") == 0)
    {
        __result = contrast_he_dark_tab.GetInfo(&__path[__pos], __info);
    }

    // hist_smoothing_str
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("hist_smoothing_str", ParameterParserDataType::Float, (double)hist_smoothing_str));
    }
    else if (PARAMETER_STRCMP(__name, "hist_smoothing_str") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)hist_smoothing_str));
    }

    // hist_curve_smoothing_str
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("hist_curve_smoothing_str", ParameterParserDataType::Float, (double)hist_curve_smoothing_str));
    }
    else if (PARAMETER_STRCMP(__name, "hist_curve_smoothing_str") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)hist_curve_smoothing_str));
    }

    // scene_change_smoothing_str
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("scene_change_smoothing_str", ParameterParserDataType::Float, (double)scene_change_smoothing_str));
    }
    else if (PARAMETER_STRCMP(__name, "scene_change_smoothing_str") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)scene_change_smoothing_str));
    }

    // scene_change_curve_smoothing_str
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("scene_change_curve_smoothing_str", ParameterParserDataType::Float, (double)scene_change_curve_smoothing_str));
    }
    else if (PARAMETER_STRCMP(__name, "scene_change_curve_smoothing_str") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)scene_change_curve_smoothing_str));
    }

    // scene_change_hist_delta_th1
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("scene_change_hist_delta_th1", ParameterParserDataType::Float, (double)scene_change_hist_delta_th1));
    }
    else if (PARAMETER_STRCMP(__name, "scene_change_hist_delta_th1") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)scene_change_hist_delta_th1));
    }

    // scene_change_hist_delta_th2
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("scene_change_hist_delta_th2", ParameterParserDataType::Float, (double)scene_change_hist_delta_th2));
    }
    else if (PARAMETER_STRCMP(__name, "scene_change_hist_delta_th2") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)scene_change_hist_delta_th2));
    }

    // scene_change_lux_idx_delta_th1
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("scene_change_lux_idx_delta_th1", ParameterParserDataType::Float, (double)scene_change_lux_idx_delta_th1));
    }
    else if (PARAMETER_STRCMP(__name, "scene_change_lux_idx_delta_th1") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)scene_change_lux_idx_delta_th1));
    }

    // scene_change_lux_idx_delta_th2
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("scene_change_lux_idx_delta_th2", ParameterParserDataType::Float, (double)scene_change_lux_idx_delta_th2));
    }
    else if (PARAMETER_STRCMP(__name, "scene_change_lux_idx_delta_th2") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)scene_change_lux_idx_delta_th2));
    }

    // stats_fusion_long_str
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("stats_fusion_long_str", ParameterParserDataType::Float, (double)stats_fusion_long_str));
    }
    else if (PARAMETER_STRCMP(__name, "stats_fusion_long_str") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)stats_fusion_long_str));
    }

    // stats_fusion_medium_str
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("stats_fusion_medium_str", ParameterParserDataType::Float, (double)stats_fusion_medium_str));
    }
    else if (PARAMETER_STRCMP(__name, "stats_fusion_medium_str") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)stats_fusion_medium_str));
    }

    // stats_fusion_long_range
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("stats_fusion_long_range", ParameterParserDataType::Float, (double)stats_fusion_long_range));
    }
    else if (PARAMETER_STRCMP(__name, "stats_fusion_long_range") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)stats_fusion_long_range));
    }

    // stats_fusion_medium_range
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("stats_fusion_medium_range", ParameterParserDataType::Float, (double)stats_fusion_medium_range));
    }
    else if (PARAMETER_STRCMP(__name, "stats_fusion_medium_range") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)stats_fusion_medium_range));
    }

    // stats_fusion_short_range
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("stats_fusion_short_range", ParameterParserDataType::Float, (double)stats_fusion_short_range));
    }
    else if (PARAMETER_STRCMP(__name, "stats_fusion_short_range") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)stats_fusion_short_range));
    }

    // adjustment_face_th1
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("adjustment_face_th1", ParameterParserDataType::Float, (double)adjustment_face_th1));
    }
    else if (PARAMETER_STRCMP(__name, "adjustment_face_th1") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)adjustment_face_th1));
    }

    // adjustment_face_th2
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("adjustment_face_th2", ParameterParserDataType::Float, (double)adjustment_face_th2));
    }
    else if (PARAMETER_STRCMP(__name, "adjustment_face_th2") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)adjustment_face_th2));
    }

    // adjustment_face_smoothing_str
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("adjustment_face_smoothing_str", ParameterParserDataType::Float, (double)adjustment_face_smoothing_str));
    }
    else if (PARAMETER_STRCMP(__name, "adjustment_face_smoothing_str") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)adjustment_face_smoothing_str));
    }

    return __result;
}

bool tmc14_rgn_dataType::gtm_percentage_tabStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "gtm_percentage") == 0)
    {
        if (__index < 4)
        {
            ParameterUtils::DoubleToString((double)gtm_percentage[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 4; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)gtm_percentage[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }

    return __result;
}

bool tmc14_rgn_dataType::gtm_percentage_tabStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "gtm_percentage") == 0)
    {
        if (__index < 4)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                gtm_percentage[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 4)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    gtm_percentage[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool tmc14_rgn_dataType::gtm_percentage_tabStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // gtm_percentage
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("gtm_percentage", ParameterParserDataType::Float, true, false, 4));
    }

    return __result;
}

bool tmc14_rgn_dataType::ltm_percentage_tabStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "ltm_percentage") == 0)
    {
        if (__index < 4)
        {
            ParameterUtils::DoubleToString((double)ltm_percentage[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 4; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)ltm_percentage[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }

    return __result;
}

bool tmc14_rgn_dataType::ltm_percentage_tabStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "ltm_percentage") == 0)
    {
        if (__index < 4)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                ltm_percentage[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 4)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    ltm_percentage[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool tmc14_rgn_dataType::ltm_percentage_tabStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // ltm_percentage
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("ltm_percentage", ParameterParserDataType::Float, true, false, 4));
    }

    return __result;
}

bool tmc14_rgn_dataType::dark_boost_offset_tabStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "dark_boost_offset") == 0)
    {
        if (__index < 2)
        {
            ParameterUtils::DoubleToString((double)dark_boost_offset[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 2; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)dark_boost_offset[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }

    return __result;
}

bool tmc14_rgn_dataType::dark_boost_offset_tabStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "dark_boost_offset") == 0)
    {
        if (__index < 2)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                dark_boost_offset[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 2)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    dark_boost_offset[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool tmc14_rgn_dataType::dark_boost_offset_tabStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // dark_boost_offset
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("dark_boost_offset", ParameterParserDataType::Float, true, false, 2));
    }

    return __result;
}

bool tmc14_rgn_dataType::tone_target_tabStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "tone_target") == 0)
    {
        if (__index < 14)
        {
            ParameterUtils::DoubleToString((double)tone_target[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 14; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)tone_target[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }

    return __result;
}

bool tmc14_rgn_dataType::tone_target_tabStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "tone_target") == 0)
    {
        if (__index < 14)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                tone_target[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 14)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    tone_target[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool tmc14_rgn_dataType::tone_target_tabStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // tone_target
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("tone_target", ParameterParserDataType::Float, true, false, 14));
    }

    return __result;
}

bool tmc14_rgn_dataType::tone_anchors_tabStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "tone_anchors") == 0)
    {
        if (__index < 14)
        {
            ParameterUtils::DoubleToString((double)tone_anchors[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 14; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)tone_anchors[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }

    return __result;
}

bool tmc14_rgn_dataType::tone_anchors_tabStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "tone_anchors") == 0)
    {
        if (__index < 14)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                tone_anchors[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 14)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    tone_anchors[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool tmc14_rgn_dataType::tone_anchors_tabStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // tone_anchors
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("tone_anchors", ParameterParserDataType::Float, true, false, 14));
    }

    return __result;
}

bool tmc14_rgn_dataType::hist_conv_kernel_tabStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "hist_conv_kernel") == 0)
    {
        if (__index < 5)
        {
            ParameterUtils::DoubleToString((double)hist_conv_kernel[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 5; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)hist_conv_kernel[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }

    return __result;
}

bool tmc14_rgn_dataType::hist_conv_kernel_tabStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "hist_conv_kernel") == 0)
    {
        if (__index < 5)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                hist_conv_kernel[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 5)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    hist_conv_kernel[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool tmc14_rgn_dataType::hist_conv_kernel_tabStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // hist_conv_kernel
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("hist_conv_kernel", ParameterParserDataType::Float, true, false, 5));
    }

    return __result;
}

bool tmc14_rgn_dataType::hist_enhance_ratio_tabStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "hist_enhance_ratio") == 0)
    {
        if (__index < 7)
        {
            ParameterUtils::DoubleToString((double)hist_enhance_ratio[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 7; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)hist_enhance_ratio[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }

    return __result;
}

bool tmc14_rgn_dataType::hist_enhance_ratio_tabStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "hist_enhance_ratio") == 0)
    {
        if (__index < 7)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                hist_enhance_ratio[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 7)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    hist_enhance_ratio[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool tmc14_rgn_dataType::hist_enhance_ratio_tabStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // hist_enhance_ratio
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("hist_enhance_ratio", ParameterParserDataType::Float, true, false, 7));
    }

    return __result;
}

bool tmc14_rgn_dataType::contrast_he_bright_tabStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "contrast_he_bright") == 0)
    {
        if (__index < 2)
        {
            ParameterUtils::DoubleToString((double)contrast_he_bright[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 2; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)contrast_he_bright[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }

    return __result;
}

bool tmc14_rgn_dataType::contrast_he_bright_tabStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "contrast_he_bright") == 0)
    {
        if (__index < 2)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                contrast_he_bright[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 2)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    contrast_he_bright[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool tmc14_rgn_dataType::contrast_he_bright_tabStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // contrast_he_bright
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("contrast_he_bright", ParameterParserDataType::Float, true, false, 2));
    }

    return __result;
}

bool tmc14_rgn_dataType::contrast_he_dark_tabStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "contrast_he_dark") == 0)
    {
        if (__index < 2)
        {
            ParameterUtils::DoubleToString((double)contrast_he_dark[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 2; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)contrast_he_dark[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }

    return __result;
}

bool tmc14_rgn_dataType::contrast_he_dark_tabStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "contrast_he_dark") == 0)
    {
        if (__index < 2)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                contrast_he_dark[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 2)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    contrast_he_dark[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool tmc14_rgn_dataType::contrast_he_dark_tabStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // contrast_he_dark
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("contrast_he_dark", ParameterParserDataType::Float, true, false, 2));
    }

    return __result;
}

bool mod_tmc14_aec_dataType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "aec_trigger") == 0)
    {
        __result = aec_trigger.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "tmc14_rgn_data") == 0)
    {
        __result = tmc14_rgn_data.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool mod_tmc14_aec_dataType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "aec_trigger") == 0)
    {
        __result = aec_trigger.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "tmc14_rgn_data") == 0)
    {
        __result = tmc14_rgn_data.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool mod_tmc14_aec_dataType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // aec_trigger
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("aec_trigger", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "aec_trigger") == 0)
    {
        __result = aec_trigger.GetInfo(&__path[__pos], __info);
    }

    // tmc14_rgn_data
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("tmc14_rgn_data", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "tmc14_rgn_data") == 0)
    {
        __result = tmc14_rgn_data.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool mod_tmc14_hdr_aec_dataType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "hdr_aec_trigger") == 0)
    {
        __result = hdr_aec_trigger.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "hdr_aec_data") == 0)
    {
        __result = hdr_aec_data.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool mod_tmc14_hdr_aec_dataType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "hdr_aec_trigger") == 0)
    {
        __result = hdr_aec_trigger.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "hdr_aec_data") == 0)
    {
        __result = hdr_aec_data.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool mod_tmc14_hdr_aec_dataType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // hdr_aec_trigger
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("hdr_aec_trigger", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "hdr_aec_trigger") == 0)
    {
        __result = hdr_aec_trigger.GetInfo(&__path[__pos], __info);
    }

    // hdr_aec_data
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("hdr_aec_data", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "hdr_aec_data") == 0)
    {
        __result = hdr_aec_data.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool mod_tmc14_hdr_aec_dataType::hdr_aec_dataStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "mod_tmc14_aec_data") == 0)
    {
        if (__index < mod_tmc14_aec_dataCount)
        {
            __result = mod_tmc14_aec_data[__index].GetValue(&__path[__pos], __value, __size);
        }
    }

    return __result;
}

bool mod_tmc14_hdr_aec_dataType::hdr_aec_dataStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "mod_tmc14_aec_data") == 0)
    {
        if (__index < mod_tmc14_aec_dataCount)
        {
            __result = mod_tmc14_aec_data[__index].SetValue(&__path[__pos], __value);
        }
    }

    return __result;
}

bool mod_tmc14_hdr_aec_dataType::hdr_aec_dataStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // mod_tmc14_aec_data
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("mod_tmc14_aec_data", ParameterParserDataType::Structure, true, false, mod_tmc14_aec_dataCount));
    }
    else if (PARAMETER_STRCMP(__name, "mod_tmc14_aec_data") == 0)
    {
        if (__index < mod_tmc14_aec_dataCount)
        {
            __result = mod_tmc14_aec_data[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, mod_tmc14_aec_dataCount));
        }
    }

    return __result;
}

bool mod_tmc14_drc_gain_dataType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "drc_gain_trigger") == 0)
    {
        __result = drc_gain_trigger.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "drc_gain_data") == 0)
    {
        __result = drc_gain_data.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool mod_tmc14_drc_gain_dataType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "drc_gain_trigger") == 0)
    {
        __result = drc_gain_trigger.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "drc_gain_data") == 0)
    {
        __result = drc_gain_data.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool mod_tmc14_drc_gain_dataType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // drc_gain_trigger
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("drc_gain_trigger", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "drc_gain_trigger") == 0)
    {
        __result = drc_gain_trigger.GetInfo(&__path[__pos], __info);
    }

    // drc_gain_data
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("drc_gain_data", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "drc_gain_data") == 0)
    {
        __result = drc_gain_data.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool mod_tmc14_drc_gain_dataType::drc_gain_dataStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "mod_tmc14_hdr_aec_data") == 0)
    {
        if (__index < mod_tmc14_hdr_aec_dataCount)
        {
            __result = mod_tmc14_hdr_aec_data[__index].GetValue(&__path[__pos], __value, __size);
        }
    }

    return __result;
}

bool mod_tmc14_drc_gain_dataType::drc_gain_dataStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "mod_tmc14_hdr_aec_data") == 0)
    {
        if (__index < mod_tmc14_hdr_aec_dataCount)
        {
            __result = mod_tmc14_hdr_aec_data[__index].SetValue(&__path[__pos], __value);
        }
    }

    return __result;
}

bool mod_tmc14_drc_gain_dataType::drc_gain_dataStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // mod_tmc14_hdr_aec_data
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("mod_tmc14_hdr_aec_data", ParameterParserDataType::Structure, true, false, mod_tmc14_hdr_aec_dataCount));
    }
    else if (PARAMETER_STRCMP(__name, "mod_tmc14_hdr_aec_data") == 0)
    {
        if (__index < mod_tmc14_hdr_aec_dataCount)
        {
            __result = mod_tmc14_hdr_aec_data[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, mod_tmc14_hdr_aec_dataCount));
        }
    }

    return __result;
}

bool chromatix_tmc14_coreType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "mod_tmc14_drc_gain_data") == 0)
    {
        if (__index < mod_tmc14_drc_gain_dataCount)
        {
            __result = mod_tmc14_drc_gain_data[__index].GetValue(&__path[__pos], __value, __size);
        }
    }

    return __result;
}

bool chromatix_tmc14_coreType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "mod_tmc14_drc_gain_data") == 0)
    {
        if (__index < mod_tmc14_drc_gain_dataCount)
        {
            __result = mod_tmc14_drc_gain_data[__index].SetValue(&__path[__pos], __value);
        }
    }

    return __result;
}

bool chromatix_tmc14_coreType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // mod_tmc14_drc_gain_data
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("mod_tmc14_drc_gain_data", ParameterParserDataType::Structure, true, false, mod_tmc14_drc_gain_dataCount));
    }
    else if (PARAMETER_STRCMP(__name, "mod_tmc14_drc_gain_data") == 0)
    {
        if (__index < mod_tmc14_drc_gain_dataCount)
        {
            __result = mod_tmc14_drc_gain_data[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, mod_tmc14_drc_gain_dataCount));
        }
    }

    return __result;
}

bool chromatix_tmc14_reserveType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "use_gtm") == 0)
    {
        ParameterUtils::DoubleToString((double)use_gtm, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "use_ltm") == 0)
    {
        ParameterUtils::DoubleToString((double)use_ltm, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "curve_model") == 0)
    {
        ParameterUtils::DoubleToString((double)curve_model, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "curve_order") == 0)
    {
        ParameterUtils::DoubleToString((double)curve_order, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "tone_max_ratio") == 0)
    {
        ParameterUtils::DoubleToString((double)tone_max_ratio, __value, __size);
        __result = true;
    }

    return __result;
}

bool chromatix_tmc14_reserveType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "use_gtm") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            use_gtm = (UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "use_ltm") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            use_ltm = (UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "curve_model") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            curve_model = (UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "curve_order") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            curve_order = (UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "tone_max_ratio") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            tone_max_ratio = (float)dblValue; 
        }
    }

    return __result;
}

bool chromatix_tmc14_reserveType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // use_gtm
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("use_gtm", ParameterParserDataType::UInt32, (double)use_gtm, 0));
    }
    else if (PARAMETER_STRCMP(__name, "use_gtm") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)use_gtm, 0));
    }

    // use_ltm
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("use_ltm", ParameterParserDataType::UInt32, (double)use_ltm, 0));
    }
    else if (PARAMETER_STRCMP(__name, "use_ltm") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)use_ltm, 0));
    }

    // curve_model
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("curve_model", ParameterParserDataType::UInt32, (double)curve_model, 0));
    }
    else if (PARAMETER_STRCMP(__name, "curve_model") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)curve_model, 0));
    }

    // curve_order
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("curve_order", ParameterParserDataType::UInt32, (double)curve_order, 0));
    }
    else if (PARAMETER_STRCMP(__name, "curve_order") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)curve_order, 0));
    }

    // tone_max_ratio
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("tone_max_ratio", ParameterParserDataType::Float, (double)tone_max_ratio));
    }
    else if (PARAMETER_STRCMP(__name, "tone_max_ratio") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)tone_max_ratio));
    }

    return __result;
}

bool chromatix_tmc14Type::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enable_section") == 0)
    {
        __result = enable_section.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "parameter_version") == 0)
    {
        __result = parameter_version.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "control_variables") == 0)
    {
        __result = control_variables.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "control_method") == 0)
    {
        __result = control_method.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_tmc14_reserve") == 0)
    {
        __result = chromatix_tmc14_reserve.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_tmc14_core") == 0)
    {
        __result = chromatix_tmc14_core.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool chromatix_tmc14Type::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enable_section") == 0)
    {
        __result = enable_section.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "parameter_version") == 0)
    {
        __result = parameter_version.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "control_variables") == 0)
    {
        __result = control_variables.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "control_method") == 0)
    {
        __result = control_method.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_tmc14_reserve") == 0)
    {
        __result = chromatix_tmc14_reserve.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_tmc14_core") == 0)
    {
        __result = chromatix_tmc14_core.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool chromatix_tmc14Type::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // enable_section
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enable_section", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "enable_section") == 0)
    {
        __result = enable_section.GetInfo(&__path[__pos], __info);
    }

    // module_version
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("module_version", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.GetInfo(&__path[__pos], __info);
    }

    // parameter_version
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("parameter_version", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "parameter_version") == 0)
    {
        __result = parameter_version.GetInfo(&__path[__pos], __info);
    }

    // control_variables
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("control_variables", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "control_variables") == 0)
    {
        __result = control_variables.GetInfo(&__path[__pos], __info);
    }

    // control_method
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("control_method", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "control_method") == 0)
    {
        __result = control_method.GetInfo(&__path[__pos], __info);
    }

    // chromatix_tmc14_reserve
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("chromatix_tmc14_reserve", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_tmc14_reserve") == 0)
    {
        __result = chromatix_tmc14_reserve.GetInfo(&__path[__pos], __info);
    }

    // chromatix_tmc14_core
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("chromatix_tmc14_core", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_tmc14_core") == 0)
    {
        __result = chromatix_tmc14_core.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool chromatix_tmc14Type::enable_sectionStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "tmc_enable") == 0)
    {
        ParameterUtils::DoubleToString((double)tmc_enable, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool chromatix_tmc14Type::enable_sectionStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "tmc_enable") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            tmc_enable = (globalelements::enable_flag_type)dblValue; 
        }
    }

    return __result;
}

bool chromatix_tmc14Type::enable_sectionStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // tmc_enable
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("tmc_enable", ParameterParserDataType::Int32, (double)tmc_enable, 0));
    }
    else if (PARAMETER_STRCMP(__name, "tmc_enable") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)tmc_enable, 0));
    }

    return __result;
}

bool chromatix_tmc14Type::module_versionStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)major_revision, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)minor_revision, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)incr_revision, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool chromatix_tmc14Type::module_versionStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            major_revision = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            minor_revision = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            incr_revision = (INT32)dblValue; 
        }
    }

    return __result;
}

bool chromatix_tmc14Type::module_versionStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // major_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("major_revision", ParameterParserDataType::Int32, (double)major_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)major_revision, 0));
    }

    // minor_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("minor_revision", ParameterParserDataType::Int32, (double)minor_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)minor_revision, 0));
    }

    // incr_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("incr_revision", ParameterParserDataType::Int32, (double)incr_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)incr_revision, 0));
    }

    return __result;
}

bool chromatix_tmc14Type::parameter_versionStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "revision") == 0)
    {
        PARAMETER_STRCPY(__value, __size, revision);
        __result = true;
    }

    return __result;
}

bool chromatix_tmc14Type::parameter_versionStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const /*__value*/)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "revision") == 0)
    {
        // Cannot modify strings, risk of memory leaks or corruption
    }

    return __result;
}

bool chromatix_tmc14Type::parameter_versionStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("revision", ParameterParserDataType::String, revision));
    }
    else if (PARAMETER_STRCMP(__name, "revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::String, revision));
    }

    return __result;
}

bool chromatix_tmc14Type::control_variablesStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "control_var_type") == 0)
    {
        if (__index < 3)
        {
            ParameterUtils::DoubleToString((double)(UINT32)control_var_type[__index], __value, __size, 0);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 3; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)(UINT32)control_var_type[i], &__value[__valuePos], (INT32)(__size - __valuePos), 0);
            }
            __result = true;
        }
    }

    return __result;
}

bool chromatix_tmc14Type::control_variablesStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "control_var_type") == 0)
    {
        if (__index < 3)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                control_var_type[__index] = (control_var_typeEnum)(UINT32)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 3)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    control_var_type[__index] = (control_var_typeEnum)(UINT32)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool chromatix_tmc14Type::control_variablesStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // control_var_type
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("control_var_type", ParameterParserDataType::Enum, true, false, 3));
    }

    return __result;
}

bool chromatix_tmc14Type::control_methodStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "aec_exp_control") == 0)
    {
        ParameterUtils::DoubleToString((double)(UINT32)aec_exp_control, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "aec_hdr_control") == 0)
    {
        ParameterUtils::DoubleToString((double)(UINT32)aec_hdr_control, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool chromatix_tmc14Type::control_methodStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "aec_exp_control") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            aec_exp_control = (ispglobalelements::tuning_control_aec_type)(UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "aec_hdr_control") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            aec_hdr_control = (ispglobalelements::tuning_control_aec_hdr_type)(UINT32)dblValue; 
        }
    }

    return __result;
}

bool chromatix_tmc14Type::control_methodStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // aec_exp_control
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("aec_exp_control", ParameterParserDataType::Enum, (double)(UINT32)aec_exp_control, 0));
    }
    else if (PARAMETER_STRCMP(__name, "aec_exp_control") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Enum, (double)(UINT32)aec_exp_control, 0));
    }

    // aec_hdr_control
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("aec_hdr_control", ParameterParserDataType::Enum, (double)(UINT32)aec_hdr_control, 0));
    }
    else if (PARAMETER_STRCMP(__name, "aec_hdr_control") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Enum, (double)(UINT32)aec_hdr_control, 0));
    }

    return __result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @brief Read enum value
/// @param entry [in] Symbol table entry
/// @param enumValue [out] Enum value to read
/// @param alignment [in] Symbol table byte alignment
/// @return true if successful
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_tmc14Type::control_variablesStruct::ReadEnum_control_var_typeEnum(
    ParameterFileSymbolTableEntry* const entry,
    chromatix_tmc14Type::control_variablesStruct::control_var_typeEnum& enumValue,
    BYTE const alignment)
{
    bool result {false};

    UINT32 uintValue {entry->ReadEnum(&result, alignment)};

    if (result)
    {
        switch (uintValue)
        {
        case static_cast<UINT32>(chromatix_tmc14Type::control_variablesStruct::control_var_typeEnum::control_lux_idx):
            enumValue = chromatix_tmc14Type::control_variablesStruct::control_var_typeEnum::control_lux_idx;
            result    = true;
            break;
        case static_cast<UINT32>(chromatix_tmc14Type::control_variablesStruct::control_var_typeEnum::control_gain):
            enumValue = chromatix_tmc14Type::control_variablesStruct::control_var_typeEnum::control_gain;
            result    = true;
            break;
        case static_cast<UINT32>(chromatix_tmc14Type::control_variablesStruct::control_var_typeEnum::control_drc_gain):
            enumValue = chromatix_tmc14Type::control_variablesStruct::control_var_typeEnum::control_drc_gain;
            result    = true;
            break;
        case static_cast<UINT32>(chromatix_tmc14Type::control_variablesStruct::control_var_typeEnum::control_exp_time_ratio):
            enumValue = chromatix_tmc14Type::control_variablesStruct::control_var_typeEnum::control_exp_time_ratio;
            result    = true;
            break;
        case static_cast<UINT32>(chromatix_tmc14Type::control_variablesStruct::control_var_typeEnum::control_aec_exp_sensitivity_ratio):
            enumValue = chromatix_tmc14Type::control_variablesStruct::control_var_typeEnum::control_aec_exp_sensitivity_ratio;
            result    = true;
            break;
        case static_cast<UINT32>(chromatix_tmc14Type::control_variablesStruct::control_var_typeEnum::control_exp_gain_ratio):
            enumValue = chromatix_tmc14Type::control_variablesStruct::control_var_typeEnum::control_exp_gain_ratio;
            result    = true;
            break;
        default:
            PARAMETER_LOG_ERROR(entry->SetManager, "Invalid value(% u) converting UINT32 to chromatix_tmc14Type::control_variablesStruct::control_var_typeEnum", uintValue);
            break;
        }
    }
    return result;
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Default Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
chromatix_tmc14TypeClass::chromatix_tmc14TypeClass(
    const CHAR* const name)
    : ParameterModule(name, MAJOR, MINOR, PATCH, nullptr)
{
    PARAMETER_INITIALIZE(SymbolTableID);
    PARAMETER_INITIALIZE(enable_section);
    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(parameter_version);
    PARAMETER_INITIALIZE(control_variables);
    PARAMETER_INITIALIZE(control_method);
    PARAMETER_INITIALIZE(chromatix_tmc14_reserve);
    PARAMETER_INITIALIZE(chromatix_tmc14_core);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
chromatix_tmc14TypeClass::chromatix_tmc14TypeClass(
    const CHAR* const name,
    ModuleData* const moduleData)
    : ParameterModule(name, MAJOR, MINOR, PATCH, moduleData)
{
    PARAMETER_INITIALIZE(SymbolTableID);
    PARAMETER_INITIALIZE(enable_section);
    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(parameter_version);
    PARAMETER_INITIALIZE(control_variables);
    PARAMETER_INITIALIZE(control_method);
    PARAMETER_INITIALIZE(chromatix_tmc14_reserve);
    PARAMETER_INITIALIZE(chromatix_tmc14_core);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Destructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
chromatix_tmc14TypeClass::~chromatix_tmc14TypeClass()
{
    if (Loaded)
    {
        UnloadModule();
    }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// GetName
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
const CHAR* chromatix_tmc14TypeClass::GetName(
    const CHAR* const type)
{
    const CHAR* name = type;
    CHAR temp[ParameterFileSymbolTableEntry::TYPE_LEN + 1];
    if (PARAMETER_STRCMP(type, ModuleData::GetType(temp, "tmc14_sw")) == 0)
    {
        name = "tmc14_sw";
    }
    return name;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load tmc14_rgn_dataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool tmc14_rgn_dataType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(gtm_percentage_tab);
    PARAMETER_INITIALIZE(ltm_percentage_tab);
    PARAMETER_INITIALIZE(dark_boost_ratio);
    PARAMETER_INITIALIZE(dark_boost_offset_tab);
    PARAMETER_INITIALIZE(tone_target_tab);
    PARAMETER_INITIALIZE(tone_anchors_tab);
    PARAMETER_INITIALIZE(scale_ratio_smooth);
    PARAMETER_INITIALIZE(face_sa_target);
    PARAMETER_INITIALIZE(face_sa_adj_max);
    PARAMETER_INITIALIZE(face_sa_adj_smooth);
    PARAMETER_INITIALIZE(face_sa_score_gain);
    PARAMETER_INITIALIZE(face_sa_score_offset);
    PARAMETER_INITIALIZE(tone_bright_adj);
    PARAMETER_INITIALIZE(tone_dark_adj);
    PARAMETER_INITIALIZE(stretch_bright_str);
    PARAMETER_INITIALIZE(stretch_dark_str);
    PARAMETER_INITIALIZE(hist_supr_range_start);
    PARAMETER_INITIALIZE(hist_supr_range_end);
    PARAMETER_INITIALIZE(hist_boost_range_start);
    PARAMETER_INITIALIZE(hist_boost_range_end);
    PARAMETER_INITIALIZE(hist_avg_range_start);
    PARAMETER_INITIALIZE(hist_avg_range_end);
    PARAMETER_INITIALIZE(hist_conv_kernel_tab);
    PARAMETER_INITIALIZE(hist_clip_slope);
    PARAMETER_INITIALIZE(hist_enhance_clamp);
    PARAMETER_INITIALIZE(hist_enhance_ratio_tab);
    PARAMETER_INITIALIZE(contrast_bright_clip);
    PARAMETER_INITIALIZE(contrast_dark_adj);
    PARAMETER_INITIALIZE(contrast_he_bright_tab);
    PARAMETER_INITIALIZE(contrast_he_dark_tab);
    PARAMETER_INITIALIZE(hist_smoothing_str);
    PARAMETER_INITIALIZE(hist_curve_smoothing_str);
    PARAMETER_INITIALIZE(scene_change_smoothing_str);
    PARAMETER_INITIALIZE(scene_change_curve_smoothing_str);
    PARAMETER_INITIALIZE(scene_change_hist_delta_th1);
    PARAMETER_INITIALIZE(scene_change_hist_delta_th2);
    PARAMETER_INITIALIZE(scene_change_lux_idx_delta_th1);
    PARAMETER_INITIALIZE(scene_change_lux_idx_delta_th2);
    PARAMETER_INITIALIZE(stats_fusion_long_str);
    PARAMETER_INITIALIZE(stats_fusion_medium_str);
    PARAMETER_INITIALIZE(stats_fusion_long_range);
    PARAMETER_INITIALIZE(stats_fusion_medium_range);
    PARAMETER_INITIALIZE(stats_fusion_short_range);
    PARAMETER_INITIALIZE(adjustment_face_th1);
    PARAMETER_INITIALIZE(adjustment_face_th2);
    PARAMETER_INITIALIZE(adjustment_face_smoothing_str);

    result = gtm_percentage_tab.Load(entry, alignment);
    if (result)
    {
        result = ltm_percentage_tab.Load(entry, alignment);
    }
    SYMBOL_TABLE_ENTRY_READ(entry, result, dark_boost_ratio, alignment)
    if (result)
    {
        result = dark_boost_offset_tab.Load(entry, alignment);
    }
    if (result)
    {
        result = tone_target_tab.Load(entry, alignment);
    }
    if (result)
    {
        result = tone_anchors_tab.Load(entry, alignment);
    }
    SYMBOL_TABLE_ENTRY_READ(entry, result, scale_ratio_smooth, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, face_sa_target, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, face_sa_adj_max, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, face_sa_adj_smooth, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, face_sa_score_gain, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, face_sa_score_offset, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, tone_bright_adj, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, tone_dark_adj, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, stretch_bright_str, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, stretch_dark_str, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, hist_supr_range_start, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, hist_supr_range_end, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, hist_boost_range_start, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, hist_boost_range_end, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, hist_avg_range_start, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, hist_avg_range_end, alignment)
    if (result)
    {
        result = hist_conv_kernel_tab.Load(entry, alignment);
    }
    SYMBOL_TABLE_ENTRY_READ(entry, result, hist_clip_slope, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, hist_enhance_clamp, alignment)
    if (result)
    {
        result = hist_enhance_ratio_tab.Load(entry, alignment);
    }
    SYMBOL_TABLE_ENTRY_READ(entry, result, contrast_bright_clip, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, contrast_dark_adj, alignment)
    if (result)
    {
        result = contrast_he_bright_tab.Load(entry, alignment);
    }
    if (result)
    {
        result = contrast_he_dark_tab.Load(entry, alignment);
    }
    SYMBOL_TABLE_ENTRY_READ(entry, result, hist_smoothing_str, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, hist_curve_smoothing_str, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, scene_change_smoothing_str, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, scene_change_curve_smoothing_str, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, scene_change_hist_delta_th1, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, scene_change_hist_delta_th2, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, scene_change_lux_idx_delta_th1, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, scene_change_lux_idx_delta_th2, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, stats_fusion_long_str, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, stats_fusion_medium_str, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, stats_fusion_long_range, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, stats_fusion_medium_range, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, stats_fusion_short_range, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, adjustment_face_th1, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, adjustment_face_th2, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, adjustment_face_smoothing_str, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load tmc14_rgn_dataType_gtm_percentage_tabStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool tmc14_rgn_dataType::gtm_percentage_tabStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(gtm_percentage);

    result = entry->ReadArray(gtm_percentage, 4, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load tmc14_rgn_dataType_ltm_percentage_tabStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool tmc14_rgn_dataType::ltm_percentage_tabStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(ltm_percentage);

    result = entry->ReadArray(ltm_percentage, 4, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load tmc14_rgn_dataType_dark_boost_offset_tabStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool tmc14_rgn_dataType::dark_boost_offset_tabStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(dark_boost_offset);

    result = entry->ReadArray(dark_boost_offset, 2, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load tmc14_rgn_dataType_tone_target_tabStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool tmc14_rgn_dataType::tone_target_tabStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(tone_target);

    result = entry->ReadArray(tone_target, 14, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load tmc14_rgn_dataType_tone_anchors_tabStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool tmc14_rgn_dataType::tone_anchors_tabStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(tone_anchors);

    result = entry->ReadArray(tone_anchors, 14, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load tmc14_rgn_dataType_hist_conv_kernel_tabStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool tmc14_rgn_dataType::hist_conv_kernel_tabStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(hist_conv_kernel);

    result = entry->ReadArray(hist_conv_kernel, 5, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load tmc14_rgn_dataType_hist_enhance_ratio_tabStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool tmc14_rgn_dataType::hist_enhance_ratio_tabStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(hist_enhance_ratio);

    result = entry->ReadArray(hist_enhance_ratio, 7, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load tmc14_rgn_dataType_contrast_he_bright_tabStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool tmc14_rgn_dataType::contrast_he_bright_tabStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(contrast_he_bright);

    result = entry->ReadArray(contrast_he_bright, 2, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load tmc14_rgn_dataType_contrast_he_dark_tabStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool tmc14_rgn_dataType::contrast_he_dark_tabStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(contrast_he_dark);

    result = entry->ReadArray(contrast_he_dark, 2, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load mod_tmc14_aec_dataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool mod_tmc14_aec_dataType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(aec_trigger);
    PARAMETER_INITIALIZE(tmc14_rgn_data);

    result = aec_trigger.Load(entry, alignment);
    if (result)
    {
        result = tmc14_rgn_data.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load mod_tmc14_hdr_aec_dataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool mod_tmc14_hdr_aec_dataType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(hdr_aec_trigger);
    PARAMETER_INITIALIZE(hdr_aec_data);

    result = hdr_aec_trigger.Load(entry, alignment);
    if (result)
    {
        result = hdr_aec_data.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load mod_tmc14_hdr_aec_dataType_hdr_aec_dataStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool mod_tmc14_hdr_aec_dataType::hdr_aec_dataStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(mod_tmc14_aec_data);


    // Reading Array mod_tmc14_aec_data from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        mod_tmc14_aec_dataCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, mod_tmc14_aec_dataCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            mod_tmc14_aec_dataID = pointerEntry->ID;
            DupObjInfo* mod_tmc14_aec_dataDupInfo = pointerEntry->GetDupObject(mod_tmc14_aec_dataCount, file, line);
            if (mod_tmc14_aec_dataDupInfo != nullptr && mod_tmc14_aec_dataDupInfo->Object != nullptr)
            {
                mod_tmc14_aec_data = (mod_tmc14_aec_dataType*)mod_tmc14_aec_dataDupInfo->Object;
                if (mod_tmc14_aec_data == nullptr)
                {
                    if (mod_tmc14_aec_dataDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "mod_tmc14_aec_data duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            mod_tmc14_aec_dataDupInfo->File, mod_tmc14_aec_dataDupInfo->Line, mod_tmc14_aec_dataDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "mod_tmc14_aec_data duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            mod_tmc14_aec_dataDupInfo->Line, mod_tmc14_aec_dataDupInfo->RefCount);
                    }
                }
            }

            if (mod_tmc14_aec_data == nullptr)
            {
                if (result && mod_tmc14_aec_dataCount > 0)
                {
                    mod_tmc14_aec_data = PARAMETER_NEW mod_tmc14_aec_dataType[mod_tmc14_aec_dataCount];
                    (void)pointerEntry->SetDupObject(mod_tmc14_aec_data, mod_tmc14_aec_dataCount, file, line);
                }
                if (!result || mod_tmc14_aec_data == nullptr)
                {
                    if (mod_tmc14_aec_dataCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate mod_tmc14_aec_data\n");
                    }
                    mod_tmc14_aec_dataCount = 0;
                }
                for (UINT32 i = 0; result && i < mod_tmc14_aec_dataCount; i++)
                {
                    result = mod_tmc14_aec_data[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (mod_tmc14_aec_data == nullptr)
        {
            mod_tmc14_aec_dataCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load mod_tmc14_drc_gain_dataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool mod_tmc14_drc_gain_dataType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(drc_gain_trigger);
    PARAMETER_INITIALIZE(drc_gain_data);

    result = drc_gain_trigger.Load(entry, alignment);
    if (result)
    {
        result = drc_gain_data.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load mod_tmc14_drc_gain_dataType_drc_gain_dataStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool mod_tmc14_drc_gain_dataType::drc_gain_dataStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(mod_tmc14_hdr_aec_data);


    // Reading Array mod_tmc14_hdr_aec_data from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        mod_tmc14_hdr_aec_dataCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, mod_tmc14_hdr_aec_dataCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            mod_tmc14_hdr_aec_dataID = pointerEntry->ID;
            DupObjInfo* mod_tmc14_hdr_aec_dataDupInfo = pointerEntry->GetDupObject(mod_tmc14_hdr_aec_dataCount, file, line);
            if (mod_tmc14_hdr_aec_dataDupInfo != nullptr && mod_tmc14_hdr_aec_dataDupInfo->Object != nullptr)
            {
                mod_tmc14_hdr_aec_data = (mod_tmc14_hdr_aec_dataType*)mod_tmc14_hdr_aec_dataDupInfo->Object;
                if (mod_tmc14_hdr_aec_data == nullptr)
                {
                    if (mod_tmc14_hdr_aec_dataDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "mod_tmc14_hdr_aec_data duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            mod_tmc14_hdr_aec_dataDupInfo->File, mod_tmc14_hdr_aec_dataDupInfo->Line, mod_tmc14_hdr_aec_dataDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "mod_tmc14_hdr_aec_data duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            mod_tmc14_hdr_aec_dataDupInfo->Line, mod_tmc14_hdr_aec_dataDupInfo->RefCount);
                    }
                }
            }

            if (mod_tmc14_hdr_aec_data == nullptr)
            {
                if (result && mod_tmc14_hdr_aec_dataCount > 0)
                {
                    mod_tmc14_hdr_aec_data = PARAMETER_NEW mod_tmc14_hdr_aec_dataType[mod_tmc14_hdr_aec_dataCount];
                    (void)pointerEntry->SetDupObject(mod_tmc14_hdr_aec_data, mod_tmc14_hdr_aec_dataCount, file, line);
                }
                if (!result || mod_tmc14_hdr_aec_data == nullptr)
                {
                    if (mod_tmc14_hdr_aec_dataCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate mod_tmc14_hdr_aec_data\n");
                    }
                    mod_tmc14_hdr_aec_dataCount = 0;
                }
                for (UINT32 i = 0; result && i < mod_tmc14_hdr_aec_dataCount; i++)
                {
                    result = mod_tmc14_hdr_aec_data[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (mod_tmc14_hdr_aec_data == nullptr)
        {
            mod_tmc14_hdr_aec_dataCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load chromatix_tmc14_coreType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_tmc14_coreType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(mod_tmc14_drc_gain_data);


    // Reading Array mod_tmc14_drc_gain_data from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        mod_tmc14_drc_gain_dataCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, mod_tmc14_drc_gain_dataCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            mod_tmc14_drc_gain_dataID = pointerEntry->ID;
            DupObjInfo* mod_tmc14_drc_gain_dataDupInfo = pointerEntry->GetDupObject(mod_tmc14_drc_gain_dataCount, file, line);
            if (mod_tmc14_drc_gain_dataDupInfo != nullptr && mod_tmc14_drc_gain_dataDupInfo->Object != nullptr)
            {
                mod_tmc14_drc_gain_data = (mod_tmc14_drc_gain_dataType*)mod_tmc14_drc_gain_dataDupInfo->Object;
                if (mod_tmc14_drc_gain_data == nullptr)
                {
                    if (mod_tmc14_drc_gain_dataDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "mod_tmc14_drc_gain_data duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            mod_tmc14_drc_gain_dataDupInfo->File, mod_tmc14_drc_gain_dataDupInfo->Line, mod_tmc14_drc_gain_dataDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "mod_tmc14_drc_gain_data duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            mod_tmc14_drc_gain_dataDupInfo->Line, mod_tmc14_drc_gain_dataDupInfo->RefCount);
                    }
                }
            }

            if (mod_tmc14_drc_gain_data == nullptr)
            {
                if (result && mod_tmc14_drc_gain_dataCount > 0)
                {
                    mod_tmc14_drc_gain_data = PARAMETER_NEW mod_tmc14_drc_gain_dataType[mod_tmc14_drc_gain_dataCount];
                    (void)pointerEntry->SetDupObject(mod_tmc14_drc_gain_data, mod_tmc14_drc_gain_dataCount, file, line);
                }
                if (!result || mod_tmc14_drc_gain_data == nullptr)
                {
                    if (mod_tmc14_drc_gain_dataCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate mod_tmc14_drc_gain_data\n");
                    }
                    mod_tmc14_drc_gain_dataCount = 0;
                }
                for (UINT32 i = 0; result && i < mod_tmc14_drc_gain_dataCount; i++)
                {
                    result = mod_tmc14_drc_gain_data[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (mod_tmc14_drc_gain_data == nullptr)
        {
            mod_tmc14_drc_gain_dataCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load chromatix_tmc14_reserveType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_tmc14_reserveType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(use_gtm);
    PARAMETER_INITIALIZE(use_ltm);
    PARAMETER_INITIALIZE(curve_model);
    PARAMETER_INITIALIZE(curve_order);
    PARAMETER_INITIALIZE(tone_max_ratio);

    SYMBOL_TABLE_ENTRY_READ(entry, result, use_gtm, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, use_ltm, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, curve_model, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, curve_order, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, tone_max_ratio, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load chromatix_tmc14Type
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_tmc14Type::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(enable_section);
    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(parameter_version);
    PARAMETER_INITIALIZE(control_variables);
    PARAMETER_INITIALIZE(control_method);
    PARAMETER_INITIALIZE(chromatix_tmc14_reserve);
    PARAMETER_INITIALIZE(chromatix_tmc14_core);

    result = enable_section.Load(entry, alignment);
    if (result)
    {
        result = module_version.Load(entry, alignment);
    }
    if (result)
    {
        result = parameter_version.Load(entry, alignment);
    }
    if (result)
    {
        result = control_variables.Load(entry, alignment);
    }
    if (result)
    {
        result = control_method.Load(entry, alignment);
    }
    if (result)
    {
        result = chromatix_tmc14_reserve.Load(entry, alignment);
    }
    if (result)
    {
        result = chromatix_tmc14_core.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load enable_sectionStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_tmc14Type::enable_sectionStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(tmc_enable);

    SYMBOL_TABLE_ENTRY_READ(entry, result, tmc_enable, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load module_versionStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_tmc14Type::module_versionStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(major_revision);
    PARAMETER_INITIALIZE(minor_revision);
    PARAMETER_INITIALIZE(incr_revision);

    SYMBOL_TABLE_ENTRY_READ(entry, result, major_revision, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, minor_revision, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, incr_revision, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load parameter_versionStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_tmc14Type::parameter_versionStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(revision);


    // Reading String revision from symbol table
    {
        UINT32 count = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, count, alignment)
        ParameterFileSymbolTableEntry stringPointerEntryCls;
        ParameterFileSymbolTableEntry* const stringPointerEntry = &stringPointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(stringPointerEntryCls, entry, alignment);
        }
        SYMBOL_TABLE_ENTRY_READ_STRING(stringPointerEntry, result, revision, count, alignment)
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load control_variablesStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_tmc14Type::control_variablesStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(control_var_type);

    for (UINT32 i = 0; result && i < 3; i++)
    {
        result = chromatix_tmc14Type::control_variablesStruct::ReadEnum_control_var_typeEnum(entry, control_var_type[i], alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load control_methodStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_tmc14Type::control_methodStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(aec_exp_control);
    PARAMETER_INITIALIZE(aec_hdr_control);

    result = ispglobalelements::ReadEnum_tuning_control_aec_type(entry, aec_exp_control, alignment);
    if (result)
    {
        result = ispglobalelements::ReadEnum_tuning_control_aec_hdr_type(entry, aec_hdr_control, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload mod_tmc14_hdr_aec_dataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void mod_tmc14_hdr_aec_dataType::Unload(ParameterFileSymbolTable* const __table)
{
    hdr_aec_data.Unload(__table);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload mod_tmc14_hdr_aec_dataType_hdr_aec_dataStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void mod_tmc14_hdr_aec_dataType::hdr_aec_dataStruct::Unload(ParameterFileSymbolTable* const __table)
{
    if (__table != nullptr && __table->ReleaseDupObject(mod_tmc14_aec_data))
    {
        PARAMETER_DELETE[] mod_tmc14_aec_data;
    }
    mod_tmc14_aec_data = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload mod_tmc14_drc_gain_dataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void mod_tmc14_drc_gain_dataType::Unload(ParameterFileSymbolTable* const __table)
{
    drc_gain_data.Unload(__table);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload mod_tmc14_drc_gain_dataType_drc_gain_dataStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void mod_tmc14_drc_gain_dataType::drc_gain_dataStruct::Unload(ParameterFileSymbolTable* const __table)
{
    for (UINT32 i = 0; i < mod_tmc14_hdr_aec_dataCount; i++)
    {
        mod_tmc14_hdr_aec_data[i].Unload(__table);
    }
    if (__table != nullptr && __table->ReleaseDupObject(mod_tmc14_hdr_aec_data))
    {
        PARAMETER_DELETE[] mod_tmc14_hdr_aec_data;
    }
    mod_tmc14_hdr_aec_data = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload chromatix_tmc14_coreType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_tmc14_coreType::Unload(ParameterFileSymbolTable* const __table)
{
    for (UINT32 i = 0; i < mod_tmc14_drc_gain_dataCount; i++)
    {
        mod_tmc14_drc_gain_data[i].Unload(__table);
    }
    if (__table != nullptr && __table->ReleaseDupObject(mod_tmc14_drc_gain_data))
    {
        PARAMETER_DELETE[] mod_tmc14_drc_gain_data;
    }
    mod_tmc14_drc_gain_data = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload chromatix_tmc14Type
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_tmc14Type::Unload(ParameterFileSymbolTable* const __table)
{
    parameter_version.Unload(__table);
    chromatix_tmc14_core.Unload(__table);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload parameter_versionStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_tmc14Type::parameter_versionStruct::Unload(ParameterFileSymbolTable* const __table)
{
    if (__table != nullptr && __table->ReleaseDupObject(revision))
    {
        PARAMETER_DELETE[] revision;
    }
    revision = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Release the ParameterModule object
/// Requires that this object was created via a ParameterSetManager::GetModule call
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_tmc14Type::Release()
{
    ((chromatix_tmc14TypeClass*)this)->Release();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Returns the ParameterModule::Modified value
/// The Modified value is incremented when the module's data or pointer values change
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
UINT32 chromatix_tmc14Type::GetModified()
{
    return ((chromatix_tmc14TypeClass*)this)->Modified;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Create module, and return a new object
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
ParameterModule* chromatix_tmc14TypeClass::Create(
    ModuleData*    moduleData,
    DefaultModule* const defModule) const
{
    chromatix_tmc14TypeClass* cls = nullptr;

    if (PARAMETER_STRCMP(Type, moduleData->Type) == 0 && Version.Value == moduleData->Version.Value)
    {
        const UINT32 id = moduleData->DuplicateID > 0 ? moduleData->DuplicateID : moduleData->ID;

        ParameterModule* const module = defModule->Find(id);
        if (module != nullptr)
        {
            cls = (chromatix_tmc14TypeClass*)module;
        }
        else
        {
            cls = PARAMETER_NEW chromatix_tmc14TypeClass(GetName(moduleData->Type), moduleData);
            cls->SymbolTableID = id;
            defModule->AddModule((ParameterModule*)cls);
        }
    }
    else if (Version.Value != moduleData->Version.Value)
    {
        PARAMETER_LOG_ERROR(moduleData->SetManager, "Unable to create chromatix_tmc14TypeClass, binary version (%lld) does not match source version (%lld)\n",
            moduleData->Version.Value, Version.Value);
    }
    else
    {
        PARAMETER_LOG_ERROR(moduleData->SetManager, "Unable to create chromatix_tmc14TypeClass, binary name (%s) does not match source name (%s)\n",
            moduleData->Type, Type);
    }

    return (ParameterModule*)cls;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load module data, return true is successful
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_tmc14TypeClass::LoadModule(
    ParameterFileSymbolTable* const table)
{
    bool result = true;

    if (!Loaded)
    {
        BYTE const alignment = table->Alignment;
        chromatix_tmc14TypeClass* const cls = this;

        ParameterFileSymbolTableEntry  entryObj;
        ParameterFileSymbolTableEntry* const entry = &entryObj;
        // With EzTune enabled, the unused upper byte of SymbolTableID value can change, so clear it
        result = table->Find(entryObj, SymbolTableID & 0x00ffffff);

        if (!result)
        {
            PARAMETER_LOG_ERROR(entry->SetManager, "chromatix_tmc14TypeClass::LoadModule Failed to find module ID (%d) in Symbol Table\n", SymbolTableID);
        }

        if (result)
        {
            result = cls->Load(entry, alignment);
        }
        Modified++;
        Loaded = true;
    }

    if (!result)
    {
        /// Module failed to fully load, unload it
        UnloadModule();
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload module data
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_tmc14TypeClass::UnloadModule()
{
    if (Loaded)
    {
        if (Data != nullptr && Data->SymbolTable != nullptr)
        {
            Unload(Data->SymbolTable);
        }
        else
        {
            PARAMETER_LOG_ERROR(Data != nullptr ? Data->SetManager : nullptr, "UnloadModule %s called with nullptr "
                "Data->SymbolTable object\n", Name);
            Unload(nullptr);
        }
        Loaded = false;
    }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// /// Release the ParameterModule object
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_tmc14TypeClass::Release()
{
    ParameterModule::Release();
}

TMC_1_4_0_NAMESPACE_END
